
use matrix_sdk::{
    Client, ruma::api::client::session::get_login_types::v3::{LoginType, IdentityProvider}
};
use serde::{Serialize, Deserialize};
use url::Url;

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum LoginChoice {
    Password,
    Sso(Vec<IdentityProvider>)
}

#[tauri::command]
pub async fn get_login_types(srv: String) -> Result<Vec<LoginChoice>, String> {
    
    let homeserver = Url::parse(srv.as_str());

    if homeserver.is_err() {
        return Err(homeserver.unwrap_err().to_string())
    }

    let homeserver = homeserver.unwrap();

    let client = Client::new(homeserver).await;

    if client.is_err() {
        return Err(client.unwrap_err().to_string());
    }
    let client: Client = client.unwrap();

    let login_types = client.get_login_types().await;

    if login_types.is_err() {
        return Err(login_types.unwrap_err().to_string())
    }

    let login_types = login_types.unwrap().flows;

    let mut choices = Vec::new();
    
    for login_type in login_types {
        match login_type {
            LoginType::Password(_) => {
                choices.push(LoginChoice::Password)
            },
            LoginType::Sso(sso) => {
                choices.push(LoginChoice::Sso(sso.identity_providers.clone()))
            }
            _ => {}
        }
    }

    Ok(choices)
}
